package com.jdbernard.teammaker

import groovy.beans.Bindable
import org.slf4j.Logger
import org.slf4j.LoggerFactory

import static java.awt.event.MouseEvent.BUTTON1 as LEFT_BUTTON
import static java.awt.event.MouseEvent.BUTTON2 as RIGHT_BUTTON

public class PlayerPanel {

    /* ======== MODEL ======== */

    static def swing = ProfessionalPickup.swing
    static def icons = [:]

    TeamPanel teamPanel
    Logger log = LoggerFactory.getLogger(getClass())

    // GUI Components
    class View {
        def panel
        def nameLabel
        def twoPtrButton
        def twoPtAttemptsButton
        def threePtrButton
        def threePtAttemptsButton
        def rebsButton
        def toversButton
    }

    View view = new View()

    class Model {
        @Bindable Player player
        @Bindable Game game
        @Bindable Game.Team team
    }

    Model model = new Model()

    /* ======== VIEW ======== */

    private void initGUI() {
        view.panel = swing.hbox {
            hstrut(5)
            view.nameLabel = label(model.player.name)
            hstrut(5)
            view.twoPtrButton = button(
                icon: getIcon("plus${Game.BASE_POINT_VALUE}"),
                iconTextGap: 0, margin: [0, 0, 0, 0],
                actionPerformed: { evt ->
                    if (LEFT_BUTTON == evt.button) {
                        model.game.twoPointers[(model.player.name)]++
                        model.game.twoPointAttemps[(model.player.name)]++
                    } else {
                        model.game.twoPointers[(model.player.name)]--
                        model.game.twoPointAttemps[(model.player.name)]--
                    }},
                toolTipText: "Click to add a successful " +
                    Game.BASE_POINT_VALUE + " point shot.\nRight-click to " +
                    "undo (remove the shot).")
            hstrut(5)
            view.threePtrButton = button(
                icon: getIcon("plus${Game.BASE_POINT_VALUE + 1}"),
                iconTextGap: 0, margin: [0, 0, 0, 0],
                actionPerformed: { evt ->
                    if (LEFT_BUTTON == evt.button) {
                        model.game.threePointers[(model.player.name)]++
                        model.game.threePointAttempts[(model.player.name)]++
                    } else {
                        model.game.threePointers[(model.player.name)]--
                        model.game.threePointAttempts[(model.player.name)]--
                    }},
                toolTipText: "Click to add a successful " + 
                    (Game.BASE_POINT_VALUE + 1) + " point shot.\nRight-click " +
                    "to undo (remove the shot).")
            hstrut(5)
            view.rebsButton = button(
                icon: getIcon("rebound"),
                iconTextGap: 0, margin: [0, 0, 0, 0],
                actionPerformed: { evt ->
                    if (LEFT_BUTTON == evt.button)
                        model.game.rebounds[(model.player.name)]++
                    else model.game.rebounds[(model.player.name)]-- },
                toolTipText: "Click to add a rebound.\nRight-click to undo.")
            hstrut(5)
            view.toversButton = button(
                icon: getIcon("turnover"),
                iconTextGap: 0, margin: [0, 0, 0, 0],
                actionPerformed: { evt ->
                    if (LEFT_BUTTON == evt.button)
                        model.game.turnovers[(model.player.name)]++
                    else model.game.turnovers[(model.player.name)]-- },
                toolTipText: "Click to add a turnover.\nRight click to undo.")
            hglue()
        }
    }

    private def getIcon(String type) {
        String iconKey = type + model.team.suffix

        if (!icons[iconKey]) {
            
            if (log.isDebugEnabled())
                log.debug("Looking for icon resources: /${iconKey}.png")

            icons[iconKey] = swing.imageIcon("/${iconKey}.png")
        }

        return icons[iconKey]
    }
    
    /* ======== CONTROLLER ======== */

    public PlayerPanel(TeamPanel teamPanel, Player player) {
        this.teamPanel = teamPanel
        model.player = player
        model.game = teamPanel.model.game
        model.team = teamPanel.model.team

        initGUI()
    }
}
